@@ -12,7 +12,7 @@ use warnings;
 # It is provided simply to keep CPAN happy:
 #   cpan -i IO::Async
 
-our $VERSION = '0.46_002';
+our $VERSION = '0.51_001';
 
 =head1 NAME
 
@@ -147,10 +147,8 @@ L<IO::Async::Loop::Poll> uses an L<IO::Poll> object for this test.
 
 L<IO::Async::Loop::Select> uses the C<select(2)> syscall.
 
-Other subclasses of loop may appear on CPAN under their own dists; such
-as L<IO::Async::Loop::Glib> which acts as a proxy for the C<Glib::MainLoop> of
-a L<Glib>-based program, or L<IO::Async::Loop::Ppoll> which uses the
-L<IO::Ppoll> object to handle signals safely on Linux.
+Other subclasses of loop may appear on CPAN under their own dists; see the
+L</SEE ALSO> section below for more detail.
 
 As well as these general-purpose classes, the L<IO::Async::Loop> constructor
 also supports looking for OS-specific subclasses, in case a more efficient
@@ -164,6 +162,14 @@ typical C<fork(2)>/C<exec(2)> style of starting child processes, and
 C<run_child> provide a method similar to perl's C<readpipe> (which is used
 to implement backticks C<``>).
 
+=head2 File Change Watches
+
+The L<IO::Async::File> object observes changes to C<stat(2)> properties of a
+file, directory, or other filesystem object. It invokes callbacks when
+properties change. This is used by L<IO::Async::FileStream> which presents
+the same events as a C<IO::Async::Stream> but operates on a regular file on
+the filesystem, observing it for updates.
+
 =head2 Asynchronous Co-routines and Functions
 
 The C<IO::Async> framework generally provides mechanisms for multiplexing IO
@@ -196,6 +202,24 @@ L<IO::Async::Channel> objects. It can be used to solve any sort of problem
 involving keeping a possibly-stateful co-routine running alongside the rest of
 an asynchronous program.
 
+=head2 Tasks
+
+A Task object represents a single outstanding action that is yet to complete,
+such as a name resolution operation or a socket connection. It stands in
+contrast to a C<IO::Async::Notifier>, which is an object that represents an
+ongoing source of activity, such as a readable filehandle of bytes or a POSIX
+signal.
+
+Tasks are a recent addition to the C<IO::Async> API and details are still
+subject to change and experimentation. Currently a Task object is an instance
+of L<CPS::Future>, but it is possible this may change in future (pardon the
+pun ;) ) to being some class within the C<IO::Async::> namespace, and having
+a reference to the containing Loop object.
+
+In general, methods that support Tasks return a new Task object to represent
+the outstanding operation. If callback functions are supplied as well, these
+will be fired in addition to the Task object becoming ready.
+
 =head2 Networking
 
 The L<IO::Async::Loop> provides several methods for performing network-based
@@ -267,6 +291,70 @@ Bugs may be reported via RT at
 Support by IRC may also be found on F<irc.perl.org> in the F<#io-async>
 channel.
 
+=cut
+
+=head1 SEE ALSO
+
+As well as the two loops supplied in this distribution, many more exist on
+CPAN. At the time of writing this includes:
+
+=over 4
+
+=item *
+
+L<IO::Async::Loop::AnyEvent> - use IO::Async with AnyEvent
+
+=item *
+
+L<IO::Async::Loop::Epoll> - use IO::Async with epoll on Linux
+
+=item *
+
+L<IO::Async::Loop::EV> - use IO::Async with EV
+
+=item *
+
+L<IO::Async::Loop::Glib> - use IO::Async with Glib or GTK
+
+=item *
+
+L<IO::Async::Loop::KQueue> - use IO::Async with kqueue
+
+=item *
+
+L<IO::Async::Loop::Mojo> - use IO::Async with Mojolicious
+
+=item *
+
+L<IO::Async::Loop::POE> - use IO::Async with POE
+
+=item *
+
+L<IO::Async::Loop::Ppoll> - use IO::Async with ppoll(2)
+
+=back
+
+Additionally, some other event loops or modules also support being run on top
+of C<IO::Async>:
+
+=over 4
+
+=item *
+
+L<AnyEvent::Impl::IOAsync> - AnyEvent adapter for IO::Async
+
+=item * 
+
+L<Gungho::Engine::IO::Async> - IO::Async Engine
+
+=item *
+
+L<POE::Loop::IO_Async> - IO::Async event loop support for POE
+
+=back
+
+=cut
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
