@@ -8,12 +8,14 @@ package IO::Async::ChildManager;
 use strict;
 use warnings;
 
-our $VERSION = '0.46_002';
+our $VERSION = '0.51_001';
 
 # Not a notifier
 
 use IO::Async::Stream;
 
+use IO::Async::OS;
+
 use Carp;
 use Scalar::Util qw( weaken );
 
@@ -70,7 +72,7 @@ This object is used indirectly via an C<IO::Async::Loop>:
     },
  );
 
- my ( $pipeRd, $pipeWr ) = $loop->pipepair;
+ my ( $pipeRd, $pipeWr ) = IO::Async::OS->pipepair;
  $loop->spawn_child(
     command => "/usr/bin/my-command",
 
@@ -274,7 +276,7 @@ sub spawn_child
       # fiddling with F_GETFL and F_SETFL (e.g. MSWin32)
       local $^F = -1;
 
-      ( $readpipe, $writepipe ) = $loop->pipepair or croak "Cannot pipe() - $!";
+      ( $readpipe, $writepipe ) = IO::Async::OS->pipepair or croak "Cannot pipe() - $!";
    }
 
    if( defined $command ) {
