@@ -9,11 +9,12 @@ use strict;
 use warnings;
 use base qw( IO::Async::Handle );
 
-our $VERSION = '0.46_002';
+our $VERSION = '0.51_001';
 
 use IO::Async::Handle;
+use IO::Async::OS;
 
-use POSIX qw( EAGAIN );
+use Errno qw( EAGAIN EWOULDBLOCK );
 
 use Socket qw(
    sockaddr_family
@@ -119,7 +120,8 @@ This is most useful for C<SOCK_DGRAM> or C<SOCK_RAW> sockets.
 =head2 on_accept_error $socket, $errno
 
 Optional. Invoked if the C<accept> syscall indicates an error (other than
-C<EAGAIN>). If not provided, failures of C<accept> will simply be ignored.
+C<EAGAIN> or C<EWOULDBLOCK>). If not provided, failures of C<accept> will
+simply be ignored.
 
 =cut
 
@@ -218,7 +220,10 @@ sub on_read_ready
          die "ARG! Missing on_accept,on_stream,on_socket!";
       }
    }
-   elsif( $! != EAGAIN ) {
+   elsif( $! == EAGAIN or $! == EWOULDBLOCK ) {
+      # ignore
+   }
+   else {
       $self->maybe_invoke_event( on_accept_error => $socket, $! );
    }
 }
@@ -300,7 +305,7 @@ is returned by the C<getaddrinfo> named resolver.
 
 Shortcut for passing a single address to listen on; it may be passed directly
 with this key, instead of in another array of its own. This should be in a
-format recognised by L<IO::Async::Loop>'s C<extract_addrinfo> method. See also
+format recognised by L<IO::Async::OS>'s C<extract_addrinfo> method. See also
 the C<EXAMPLES> section.
 
 =back
@@ -446,11 +451,11 @@ sub listen
       my ( $listenerr, $binderr, $sockopterr, $socketerr );
 
       foreach my $addr ( @$addrlist ) {
-         my ( $family, $socktype, $proto, $address ) = $loop->extract_addrinfo( $addr );
+         my ( $family, $socktype, $proto, $address ) = IO::Async::OS->extract_addrinfo( $addr );
 
          my $sock;
 
-         unless( $sock = $loop->socket( $family, $socktype, $proto ) ) {
+         unless( $sock = IO::Async::OS->socket( $family, $socktype, $proto ) ) {
             $socketerr = $!;
             $on_fail->( socket => $family, $socktype, $proto, $! ) if $on_fail;
             next;
@@ -583,7 +588,7 @@ sockets.
 =head2 Passing Plain Socket Addresses
 
 The C<addr> or C<addrs> parameters should contain a definition of a plain
-socket address in a form that the L<IO::Async::Loop> C<extract_addrinfo>
+socket address in a form that the L<IO::Async::OS> C<extract_addrinfo>
 method can use.
 
 This example shows how to use the C<Socket> functions to construct one for
