@@ -19,12 +19,15 @@ use Test::Refcount;
 
 use IO::Async::Test qw();
 
+use IO::Async::OS;
+
 use IO::File;
 use Fcntl qw( SEEK_SET );
 use POSIX qw( SIGTERM WIFEXITED WEXITSTATUS WIFSIGNALED WTERMSIG );
+use Socket qw( sockaddr_family AF_UNIX );
 use Time::HiRes qw( time );
 
-our $VERSION = '0.46_002';
+our $VERSION = '0.51_001';
 
 # Abstract Units of Time
 use constant AUT => $ENV{TEST_QUICK_TIMERS} ? 0.1 : 1;
@@ -158,7 +161,7 @@ use constant count_tests_io => 16;
 sub run_tests_io
 {
    {
-      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot create socket pair - $!";
+      my ( $S1, $S2 ) = IO::Async::OS->socketpair or die "Cannot create socket pair - $!";
       $_->blocking( 0 ) for $S1, $S2;
 
       my $readready  = 0;
@@ -224,7 +227,7 @@ sub run_tests_io
 
    # HUP of pipe - can be different to sockets on some architectures
    {
-      my ( $Prd, $Pwr ) = $loop->pipepair or die "Cannot pipepair - $!";
+      my ( $Prd, $Pwr ) = IO::Async::OS->pipepair or die "Cannot pipepair - $!";
       $_->blocking( 0 ) for $Prd, $Pwr;
 
       my $readready = 0;
@@ -253,25 +256,29 @@ sub run_tests_io
    SKIP: {
       $loop->_CAN_ON_HANGUP or skip "Loop cannot watch_io for on_hangup", 2;
 
-      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot socketpair - $!";
-      $_->blocking( 0 ) for $S1, $S2;
+      SKIP: {
+         my ( $S1, $S2 ) = IO::Async::OS->socketpair or die "Cannot socketpair - $!";
+         $_->blocking( 0 ) for $S1, $S2;
 
-      my $hangup = 0;
-      $loop->watch_io(
-         handle => $S1,
-         on_hangup => sub { $hangup = 1 },
-      );
+         sockaddr_family( $S1->sockname ) == AF_UNIX or skip "Cannot reliably detect hangup condition on non AF_UNIX sockets", 1;
 
-      $S2->close;
+         my $hangup = 0;
+         $loop->watch_io(
+            handle => $S1,
+            on_hangup => sub { $hangup = 1 },
+         );
 
-      $loop->loop_once( 0.1 );
+         $S2->close;
 
-      is( $hangup, 1, '$hangup after socket close' );
+         $loop->loop_once( 0.1 );
 
-      my ( $Prd, $Pwr ) = $loop->pipepair or die "Cannot pipepair - $!";
+         is( $hangup, 1, '$hangup after socket close' );
+      }
+
+      my ( $Prd, $Pwr ) = IO::Async::OS->pipepair or die "Cannot pipepair - $!";
       $_->blocking( 0 ) for $Prd, $Pwr;
 
-      $hangup = 0;
+      my $hangup = 0;
       $loop->watch_io(
          handle => $Pwr,
          on_hangup => sub { $hangup = 1 },
@@ -286,7 +293,7 @@ sub run_tests_io
 
    # Check that combined read/write handlers can cancel each other
    {
-      my ( $S1, $S2 ) = $loop->socketpair or die "Cannot socketpair - $!";
+      my ( $S1, $S2 ) = IO::Async::OS->socketpair or die "Cannot socketpair - $!";
       $_->blocking( 0 ) for $S1, $S2;
 
       my $callcount = 0;
@@ -312,7 +319,7 @@ sub run_tests_io
    # Check that error conditions that aren't true read/write-ability are still
    # invoked
    {
-      my ( $S1, $S2 ) = $loop->socketpair( 'inet', 'dgram' ) or die "Cannot create AF_INET/SOCK_DGRAM connected pair - $!";
+      my ( $S1, $S2 ) = IO::Async::OS->socketpair( 'inet', 'dgram' ) or die "Cannot create AF_INET/SOCK_DGRAM connected pair - $!";
       $_->blocking( 0 ) for $S1, $S2;
       $S2->close;
 
@@ -368,10 +375,80 @@ Tests the Loop's ability to handle timer events
 
 =cut
 
-use constant count_tests_timer => 12;
+use constant count_tests_timer => 21;
 sub run_tests_timer
 {
    my $done = 0;
+   # New watch/unwatch API
+
+   cmp_ok( abs( $loop->time - time ), "<", 0.1, '$loop->time gives the current time' );
+
+   $loop->watch_time( after => 2 * AUT, code => sub { $done = 1; } );
+
+   is_oneref( $loop, '$loop has refcount 1 after watch_time' );
+
+   time_between {
+      my $now = time;
+      $loop->loop_once( 5 * AUT );
+
+      # poll might have returned just a little early, such that the TimerQueue
+      # doesn't think anything is ready yet. We need to handle that case.
+      while( !$done ) {
+         die "It should have been ready by now" if( time - $now > 5 * AUT );
+         $loop->loop_once( 0.1 * AUT );
+      }
+   } 1.5, 2.5, 'loop_once(5) while waiting for watch_time after';
+
+   $loop->watch_time( at => time + 2 * AUT, code => sub { $done = 2; } );
+
+   time_between {
+      my $now = time;
+      $loop->loop_once( 5 * AUT );
+
+      # poll might have returned just a little early, such that the TimerQueue
+      # doesn't think anything is ready yet. We need to handle that case.
+      while( !$done ) {
+         die "It should have been ready by now" if( time - $now > 5 * AUT );
+         $loop->loop_once( 0.1 * AUT );
+      }
+   } 1.5, 2.5, 'loop_once(5) while waiting for watch_time at';
+
+   my $cancelled_fired = 0;
+   my $id = $loop->watch_time( after => 1 * AUT, code => sub { $cancelled_fired = 1 } );
+   $loop->unwatch_time( $id );
+   undef $id;
+
+   $loop->loop_once( 2 * AUT );
+
+   ok( !$cancelled_fired, 'unwatched watch_time does not fire' );
+
+   $loop->watch_time( after => -1, code => sub { $done = 1 } );
+
+   $done = 0;
+
+   time_between {
+      $loop->loop_once while !$done;
+   } 0, 0.1, 'loop_once while waiting for negative interval timer';
+
+   {
+      my $done;
+
+      my $id;
+      $id = $loop->watch_time( after => 1 * AUT, code => sub {
+         $loop->unwatch_time( $id ); undef $id;
+      });
+
+      $loop->watch_time( after => 1.1 * AUT, code => sub {
+         $done++;
+      });
+
+      wait_for { $done };
+
+      is( $done, 1, 'Other timers still fire after self-cancelling one' );
+   }
+
+   # Legacy enqueue/requeue/cancel API
+   $done = 0;
 
    $loop->enqueue_timer( delay => 2 * AUT, code => sub { $done = 1; } );
 
@@ -389,8 +466,29 @@ sub run_tests_timer
       }
    } 1.5, 2.5, 'loop_once(5) while waiting for timer';
 
-   my $cancelled_fired = 0;
-   my $id = $loop->enqueue_timer( delay => 1 * AUT, code => sub { $cancelled_fired = 1 } );
+   SKIP: {
+      skip "Unable to handle sub-second timers accurately", 3 unless $loop->_CAN_SUBSECOND_ACCURATELY;
+
+      # Check that short delays are achievable in one ->loop_once call
+      foreach my $delay ( 0.001, 0.01, 0.1 ) {
+         my $done;
+         my $count = 0;
+         my $start = time;
+
+         $loop->enqueue_timer( delay => $delay, code => sub { $done++ } );
+
+         while( !$done ) {
+            $loop->loop_once( 1 );
+            $count++;
+            last if time - $start > 5; # bailout
+         }
+
+         is( $count, 1, "One ->loop_once(1) sufficient for a single $delay second timer" );
+      }
+   }
+
+   $cancelled_fired = 0;
+   $id = $loop->enqueue_timer( delay => 1 * AUT, code => sub { $cancelled_fired = 1 } );
    $loop->cancel_timer( $id );
    undef $id;
 
@@ -420,29 +518,6 @@ sub run_tests_timer
    } 1.5, 2.5, 'requeued timer of delay 2';
 
    is( $done, 2, '$done is 2 after requeued timer' );
-
-   $loop->enqueue_timer( delay => -1, code => sub { $done = 1 } );
-
-   $done = 0;
-
-   time_between {
-      $loop->loop_once while !$done;
-   } 0, 0.1, 'loop_once while waiting for negative interval timer';
-
-   my $doneA;
-   my $doneB;
-
-   $id = $loop->enqueue_timer( delay => 1 * AUT, code => sub {
-      $loop->cancel_timer( $id ); undef $id;
-      $doneA++;
-   });
-
-   $loop->enqueue_timer( delay => 1.1 * AUT, code => sub { $doneB++ } );
-
-   $loop->loop_once( 1 * AUT ) for 1 .. 3;
-
-   is( $doneA, 1, 'Self-cancelling timer still fires' );
-   is( $doneB, 1, 'Other timers still fire after self-cancelling one' );
 }
 
 =head2 signal
@@ -524,7 +599,7 @@ Tests the Loop's support for idle handlers
 
 =cut
 
-use constant count_tests_idle => 10;
+use constant count_tests_idle => 11;
 sub run_tests_idle
 {
    my $called = 0;
@@ -561,11 +636,20 @@ sub run_tests_idle
 
    is( $called, 2, 'unwatched deferral not called' );
 
-   $loop->later( sub { $called = 3 } );
+   $id = $loop->watch_idle( when => 'later', code => sub { $called = 3 } );
+   my $timer_id = $loop->watch_time( after => 5, code => sub {} );
 
    $loop->loop_once( 1 );
 
-   is( $called, 3, '$loop->later shortcut works' );
+   is( $called, 3, '$loop->later still invoked with enqueued timer' );
+
+   $loop->unwatch_time( $timer_id );
+
+   $loop->later( sub { $called = 4 } );
+
+   $loop->loop_once( 1 );
+
+   is( $called, 4, '$loop->later shortcut works' );
 }
 
 =head2 child
@@ -651,7 +735,7 @@ sub run_tests_control
 
    time_between { $loop->loop_once( 2 * AUT ) } 1.5, 2.5, 'loop_once(2) when idle';
 
-   $loop->enqueue_timer( delay => 0.1, code => sub { $loop->stop( result => "here" ) } );
+   $loop->watch_time( after => 0.1, code => sub { $loop->stop( result => "here" ) } );
 
    local $SIG{ALRM} = sub { die "Test timed out before ->stop" };
    alarm( 1 );
@@ -662,14 +746,14 @@ sub run_tests_control
 
    is_deeply( \@result, [ result => "here" ], '->stop arguments returned by ->run' );
 
-   $loop->enqueue_timer( delay => 0.1, code => sub { $loop->stop( result => "here" ) } );
+   $loop->watch_time( after => 0.1, code => sub { $loop->stop( result => "here" ) } );
 
    my $result = $loop->run;
 
    is( $result, "result", 'First ->stop argument returned by ->run in scalar context' );
 
-   $loop->enqueue_timer( delay => 0.1, code => sub {
-      $loop->enqueue_timer( delay => 0.1, code => sub { $loop->stop( "inner" ) } );
+   $loop->watch_time( after => 0.1, code => sub {
+      $loop->watch_time( after => 0.1, code => sub { $loop->stop( "inner" ) } );
       my @result = $loop->run;
       $loop->stop( @result, "outer" );
    } );
@@ -678,7 +762,7 @@ sub run_tests_control
 
    is_deeply( \@result, [ "inner", "outer" ], '->run can be nested properly' );
 
-   $loop->enqueue_timer( delay => 0.1, code => sub { $loop->loop_stop } );
+   $loop->watch_time( after => 0.1, code => sub { $loop->loop_stop } );
 
    local $SIG{ALRM} = sub { die "Test timed out before ->loop_stop" };
    alarm( 1 );
