@@ -9,7 +9,9 @@ use strict;
 use warnings;
 use base qw( IO::Async::Function );
 
-our $VERSION = '0.46_002';
+our $VERSION = '0.51_001';
+
+use CPS::Future;
 
 use Socket 1.93 qw(
    AI_NUMERICHOST AI_PASSIVE
@@ -24,12 +26,7 @@ BEGIN {
    *_getnameinfo = \&Socket::getnameinfo;
 }
 
-BEGIN {
-   # More cheating
-   require IO::Async::Loop;
-   *_getfamilybyname   = \&IO::Async::Loop::_getfamilybyname;
-   *_getsocktypebyname = \&IO::Async::Loop::_getsocktypebyname;
-}
+use IO::Async::OS;
 
 use Time::HiRes qw( alarm );
 
@@ -169,6 +166,11 @@ passed the exception thrown by the function.
 
 =back
 
+=head2 $task = $resolver->resolve( %params )
+
+When returning a task, the C<on_resolved> and C<on_error> continuations are
+optional.
+
 =cut
 
 sub resolve
@@ -187,19 +189,32 @@ sub resolve
 
    exists $METHODS{$type} or croak "Expected 'type' to be an existing resolver method, got '$type'";
 
-   my $on_resolved = $args{on_resolved};
-   ref $on_resolved or croak "Expected 'on_resolved' to be a reference";
+   my $on_resolved;
+   if( $on_resolved = $args{on_resolved} ) {
+      ref $on_resolved or croak "Expected 'on_resolved' to be a reference";
+   }
+   elsif( !defined wantarray ) {
+      croak "Expected 'on_resolved' or to return a Task";
+   }
 
-   my $on_error = $args{on_error};
-   ref $on_error or croak "Expected 'on_error' to be a reference";
+   my $on_error;
+   if( $on_error = $args{on_error} ) {
+      ref $on_error or croak "Expected 'on_error' to be a reference";
+   }
+   elsif( !defined wantarray ) {
+      croak "Expected 'on_error' or to return a Task";
+   }
 
    my $timeout = $args{timeout} || 10;
 
-   $self->call(
-      args      => [ $type, $timeout, @{$args{data}} ],
-      on_return => $on_resolved,
-      on_error  => $on_error,
+   my $task = $self->call(
+      args => [ $type, $timeout, @{$args{data}} ],
    );
+
+   $task->on_done( $on_resolved ) if $on_resolved;
+   $task->on_fail( $on_error    ) if $on_error;
+
+   return $task;
 }
 
 =head2 $resolver->getaddrinfo( %args )
@@ -264,6 +279,11 @@ like an IPv4 or IPv6 string, a synchronous lookup will first be performed
 using the C<AI_NUMERICHOST> flag. If this gives an C<EAI_NONAME> error, then
 the lookup is performed asynchronously instead.
 
+=head2 $task = $resolver->getaddrinfo( %args )
+
+When returning a task, the C<on_resolved> and C<on_error> continuations are
+optional.
+
 =cut
 
 sub getaddrinfo
@@ -271,14 +291,20 @@ sub getaddrinfo
    my $self = shift;
    my %args = @_;
 
+   $args{on_resolved} or defined wantarray or
+      croak "Expected 'on_resolved' or to return a Task";
+
+   $args{on_error} or defined wantarray or
+      croak "Expected 'on_error' or to return a Task";
+
    my $host    = $args{host}    || "";
    my $service = $args{service} || "";
    my $flags   = $args{flags}   || 0;
 
    $flags |= AI_PASSIVE if $args{passive};
 
-   $args{family}   = _getfamilybyname( $args{family} )     if defined $args{family};
-   $args{socktype} = _getsocktypebyname( $args{socktype} ) if defined $args{socktype};
+   $args{family}   = IO::Async::OS->getfamilybyname( $args{family} )     if defined $args{family};
+   $args{socktype} = IO::Async::OS->getsocktypebyname( $args{socktype} ) if defined $args{socktype};
 
    # Clear any other existing but undefined hints
    defined $args{$_} or delete $args{$_} for keys %args;
@@ -298,19 +324,23 @@ sub getaddrinfo
        );
 
        if( !$err ) {
-          $args{on_resolved}->( @results );
-          return;
+          my $task = CPS::Future->new;
+          $task->on_done( $args{on_resolved} ) if $args{on_resolved};
+          $task->done( @results );
+          return $task;
        }
        elsif( $err == EAI_NONAME ) {
           # fallthrough to async case
        }
        else {
-          $args{on_error}->( "$err\n" );
-          return;
+          my $task = CPS::Future->new;
+          $task->on_fail( $args{on_error} ) if $args{on_error};
+          $task->fail( "$err\n" );
+          return $task;
        }
    }
 
-   $self->resolve(
+   return $self->resolve(
       type    => "getaddrinfo_hash",
       data    => [
          host    => $host,
@@ -374,6 +404,11 @@ As a specific optimsation, this method will try to perform a lookup of numeric
 values synchronously, rather than asynchronously, if both the
 C<NI_NUMERICHOST> and C<NI_NUMERICSERV> flags are given.
 
+=head2 $task = $resolver->getnameinfo( %args )
+
+When returning a task, the C<on_resolved> and C<on_error> continuations are
+optional.
+
 =cut
 
 sub getnameinfo
@@ -381,8 +416,11 @@ sub getnameinfo
    my $self = shift;
    my %args = @_;
 
-   my $on_resolved = $args{on_resolved};
-   ref $on_resolved or croak "Expected 'on_resolved' to be a reference";
+   $args{on_resolved} or defined wantarray or
+      croak "Expected 'on_resolved' or to return a Task";
+
+   $args{on_error} or defined wantarray or
+      croak "Expected 'on_error' or to return a Task";
 
    my $flags = $args{flags} || 0;
 
@@ -395,24 +433,32 @@ sub getnameinfo
    if( $flags & (NI_NUMERICHOST|NI_NUMERICSERV) ) {
       # This is a numeric-only lookup that can be done synchronously
       my ( $err, $host, $service ) = _getnameinfo( $args{addr}, $flags );
+      my $task = CPS::Future->new;
+
       if( $err ) {
-         $args{on_error}->( "$err\n" );
+         $task->on_fail( $args{on_error} ) if $args{on_error};
+         $task->fail( "$err\n" );
       }
       else {
-         $on_resolved->( $host, $service );
+         $task->on_done( $args{on_resolved} ) if $args{on_resolved};
+         $task->done( $host, $service );
       }
-      return;
+
+      return $task;
    }
 
-   $self->resolve(
+   my $task = $self->resolve(
       type    => "getnameinfo",
       data    => [ $args{addr}, $flags ],
       timeout => $args{timeout},
-      on_resolved => sub {
-         $on_resolved->( @{ $_[0] } ); # unpack the ARRAY ref
-      },
-      on_error    => $args{on_error},
+   )->transform(
+      done => sub { @{ $_[0] } }, # unpack the ARRAY ref
    );
+
+   $task->on_done( $args{on_resolved} ) if $args{on_resolved};
+   $task->on_fail( $args{on_error}    ) if $args{on_error};
+
+   return $task;
 }
 
 =head1 FUNCTIONS
@@ -535,8 +581,8 @@ register_resolver getaddrinfo_hash => sub {
    my $host    = delete $args{host};
    my $service = delete $args{service};
 
-   $args{family}   = _getfamilybyname( $args{family} )     if defined $args{family};
-   $args{socktype} = _getsocktypebyname( $args{socktype} ) if defined $args{socktype};
+   $args{family}   = IO::Async::OS->getfamilybyname( $args{family} )     if defined $args{family};
+   $args{socktype} = IO::Async::OS->getsocktypebyname( $args{socktype} ) if defined $args{socktype};
 
    # Clear any other existing but undefined hints
    defined $args{$_} or delete $args{$_} for keys %args;
@@ -551,8 +597,8 @@ register_resolver getaddrinfo_hash => sub {
 register_resolver getaddrinfo_array => sub {
    my ( $host, $service, $family, $socktype, $protocol, $flags ) = @_;
 
-   $family   = _getfamilybyname( $family );
-   $socktype = _getsocktypebyname( $socktype );
+   $family   = IO::Async::OS->getfamilybyname( $family );
+   $socktype = IO::Async::OS->getsocktypebyname( $socktype );
 
    my %hints;
    $hints{family}   = $family   if defined $family;
