@@ -8,11 +8,11 @@ package IO::Async::Socket;
 use strict;
 use warnings;
 
-our $VERSION = '0.46_002';
+our $VERSION = '0.51_001';
 
 use base qw( IO::Async::Handle );
 
-use POSIX qw( EAGAIN EWOULDBLOCK EINTR );
+use Errno qw( EAGAIN EWOULDBLOCK EINTR );
 
 use Carp;
 
