@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.46_002';
+our $VERSION = '0.51_001';
 
 use Carp;
 
@@ -134,7 +134,7 @@ sub _make_enqueueargs
 {
    my $self = shift;
 
-   return time => $self->{time};
+   return at => $self->{time};
 }
 
 =head1 AUTHOR
