@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.46_002';
+our $VERSION = '0.51_001';
 
 use Carp;
 
@@ -162,7 +162,7 @@ sub _make_enqueueargs
    my $self = shift;
 
    undef $self->{expired};
-   return delay => $self->{delay};
+   return after => $self->{delay};
 }
 
 =head2 $timer->reset
@@ -180,10 +180,8 @@ sub reset
 
    return if !$self->is_running;
 
-   $self->{id} = $loop->requeue_timer(
-      $self->{id},
-      delay => $self->{delay},
-   );
+   $self->stop;
+   $self->start;
 }
 
 =head1 EXAMPLES
